@@ -8,7 +8,8 @@ from pyknp import Morpheme, Tag
 from pyknp_eventgraph.builder import Builder
 from pyknp_eventgraph.component import Component
 from pyknp_eventgraph.helper import PAS_ORDER, convert_katakana_to_hiragana
-from pyknp_eventgraph.token import Token
+from pyknp_eventgraph.base_phrase import BasePhrase
+from pyknp_eventgraph.helper import convert_mrphs_to_surf
 
 if TYPE_CHECKING:
     from pyknp_eventgraph.pas import PAS
@@ -25,8 +26,8 @@ class Argument(Component):
         eid (int): An entity ID.
         flag (str): A flag.
         sdist (int): The sentence distance between this argument and the predicate.
-        head_token (Token, optional): A head token.
-
+        arg (:class:`pyknp.knp.pas.Argument`, optional): An Argument object in pyknp.
+        head_base_phrase (Token, optional): A head token.
     """
 
     def __init__(self, pas: 'PAS', case: str, eid: int, flag: str, sdist: int, arg: Optional[PyknpArgument] = None):
@@ -36,7 +37,7 @@ class Argument(Component):
         self.flag: str = flag
         self.sdist: int = sdist
         self.arg: Optional[PyknpArgument] = arg
-        self.head_token: Optional[Token] = None
+        self.head_base_phrase: Optional[BasePhrase] = None
 
         self._surf = None
         self._normalized_surf = None
@@ -52,59 +53,72 @@ class Argument(Component):
     @property
     def tag(self) -> Optional[Tag]:
         """The tag of the head token."""
-        return self.head_token.tag
+        return self.head_base_phrase.tag
 
     @property
     def surf(self) -> str:
         """A surface string."""
         if self._surf is None:
-            self._surf = self.mrphs.replace(' ', '')
+            self._surf = convert_mrphs_to_surf(self.mrphs)
         return self._surf
 
     @property
     def normalized_surf(self) -> str:
         """A normalized surface string."""
         if self._normalized_surf is None:
-            self._normalized_surf = self.normalized_mrphs.replace(' ', '')
+            self._normalized_surf = convert_mrphs_to_surf(self.normalized_mrphs)
         return self._normalized_surf
 
     @property
     def mrphs(self) -> str:
         """A tokenized surface string."""
         if self._mrphs is None:
-            self._mrphs = self._token_to_text(self.head_token, truncate=False, include_modifiees=True)
+            self._mrphs = self._base_phrase_to_text(self.head_base_phrase, truncate=False, include_modifiees=True)
         return self._mrphs
 
     @property
     def normalized_mrphs(self) -> str:
         """A tokenized/normalized surface string."""
         if self._normalized_mrphs is None:
-            self._normalized_mrphs = self._token_to_text(self.head_token, truncate=True, include_modifiees=True)
+            self._normalized_mrphs = self._base_phrase_to_text(
+                self.head_base_phrase,
+                truncate=True,
+                include_modifiees=True
+            )
         return self._normalized_mrphs
 
     @property
     def reps(self) -> str:
         """A representative string."""
         if self._reps is None:
-            self._reps = self._token_to_text(self.head_token, mode='reps', truncate=False, include_modifiees=True)
+            self._reps = self._base_phrase_to_text(
+                self.head_base_phrase,
+                mode='reps',
+                truncate=False,
+                include_modifiees=True
+            )
         return self._reps
 
     @property
     def normalized_reps(self) -> str:
         """A normalized representative string."""
         if self._normalized_reps is None:
-            self._normalized_reps = self._token_to_text(self.head_token, mode='reps', truncate=True,
-                                                        include_modifiees=True)
+            self._normalized_reps = self._base_phrase_to_text(
+                self.head_base_phrase,
+                mode='reps',
+                truncate=True,
+                include_modifiees=True
+            )
         return self._normalized_reps
 
     @property
     def head_reps(self) -> str:
         """A head representative string."""
         if self._head_reps is None:
-            if self.head_token.tag:  # Not an exophora.
-                head_reps = self.head_token.tag.head_prime_repname or self.head_token.tag.head_repname
+            if self.head_base_phrase.tag:  # Not an exophora.
+                head_reps = self.head_base_phrase.tag.head_prime_repname or self.head_base_phrase.tag.head_repname
                 if head_reps:
-                    self._head_reps = f'[{head_reps}]' if self.head_token.omitted_case else head_reps
+                    self._head_reps = f'[{head_reps}]' if self.head_base_phrase.omitted_case else head_reps
             self._head_reps = self._head_reps or self.normalized_reps
         return self._head_reps
 
@@ -113,7 +127,8 @@ class Argument(Component):
         """A list of IDs of events modifying this predicate (adnominal)."""
         if self._adnominal_event_ids is None:
             self._adnominal_event_ids = sorted(
-                event.evid for t in self.head_token.modifiees(include_self=True) for event in t.adnominal_events
+                event.evid for t in self.head_base_phrase.modifiees(include_self=True)
+                for event in t.adnominal_events
             )
         return self._adnominal_event_ids
 
@@ -122,7 +137,7 @@ class Argument(Component):
         """A list of IDs of events modifying this predicate (sentential complement)."""
         if self._sentential_complement_event_ids is None:
             self._sentential_complement_event_ids = sorted(
-                event.evid for t in self.head_token.modifiees(include_self=True)
+                event.evid for t in self.head_base_phrase.modifiees(include_self=True)
                 for event in t.sentential_complement_events
             )
         return self._sentential_complement_event_ids
@@ -132,47 +147,53 @@ class Argument(Component):
         """A list of child words."""
         if self._children is None:
             self._children = []
-            for token in reversed(self.head_token.modifiers()):
+            for bp in reversed(self.head_base_phrase.modifiers()):
                 self._children.append({
-                    'surf': self._token_to_text(token, mode='mrphs', truncate=False).replace(' ', ''),
-                    'normalized_surf': self._token_to_text(token, mode='mrphs', truncate=True).replace(' ', ''),
-                    'mrphs': self._token_to_text(token, mode='mrphs', truncate=False),
-                    'normalized_mrphs': self._token_to_text(token, mode='mrphs', truncate=True),
-                    'reps': self._token_to_text(token, mode='reps', truncate=False),
-                    'normalized_reps': self._token_to_text(token, mode='reps', truncate=True),
-                    'adnominal_event_ids': [event.evid for event in token.adnominal_events],
-                    'sentential_complement_event_ids': [event.evid for event in token.sentential_complement_events],
-                    'modifier': '修飾' in token.tag.features,
-                    'possessive': token.tag.features.get('係', '') == 'ノ格',
+                    'surf': convert_mrphs_to_surf(self._base_phrase_to_text(bp, mode='mrphs', truncate=False)),
+                    'normalized_surf': convert_mrphs_to_surf(
+                        self._base_phrase_to_text(bp, mode='mrphs', truncate=True)
+                    ),
+                    'mrphs': self._base_phrase_to_text(bp, mode='mrphs', truncate=False),
+                    'normalized_mrphs': self._base_phrase_to_text(bp, mode='mrphs', truncate=True),
+                    'reps': self._base_phrase_to_text(bp, mode='reps', truncate=False),
+                    'normalized_reps': self._base_phrase_to_text(bp, mode='reps', truncate=True),
+                    'adnominal_event_ids': [e.evid for e in bp.adnominal_events],
+                    'sentential_complement_event_ids': [e.evid for e in bp.sentential_complement_events],
+                    'modifier': '修飾' in bp.tag.features,
+                    'possessive': bp.tag.features.get('係', '') == 'ノ格',
                 })
         return self._children
 
-    def _token_to_text(self, token: Token, mode: str = 'mrphs', truncate: bool = False,
-                       include_modifiees: bool = False) -> str:
-        """Convert a token to a text.
+    def _base_phrase_to_text(
+            self,
+            bp: BasePhrase,
+            mode: str = 'mrphs',
+            truncate: bool = False,
+            include_modifiees: bool = False
+    ) -> str:
+        """Convert a base phrase to a text.
 
         Args:
-            token: A token.
+            bp: A base phrase.
             mode: A type of token representation, which can take either "mrphs" or "reps".
             truncate: If true, adjunct words are truncated.
             include_modifiees: If true, parents are used to construct a compound phrase.
-
         """
         assert mode in {'mrphs', 'reps'}
-        if token.omitted_case:
-            if token.exophora:
-                base = token.exophora
+        if bp.omitted_case:
+            if bp.exophora:
+                base = bp.exophora
             else:
-                mrphs = self._truncate_mrphs(list(token.tag.mrph_list()))
+                mrphs = self._truncate_mrphs(list(bp.tag.mrph_list()))
                 base = self._format_mrphs(mrphs, mode, normalize=True)
             case = convert_katakana_to_hiragana(self.case)
             case = case if mode == 'mrphs' else f'{case}/{case}'
             return f'[{base}]' if truncate else f'[{base} {case}]'
         else:
-            mrphs = list(token.tag.mrph_list())
+            mrphs = list(bp.tag.mrph_list())
             if include_modifiees:
-                for parent_token in token.modifiees():
-                    mrphs += (parent_token.tag.mrph_list())
+                for parent_base_phrase in bp.modifiees():
+                    mrphs += (parent_base_phrase.tag.mrph_list())
             if truncate:
                 mrphs = self._truncate_mrphs(mrphs)
                 return self._format_mrphs(mrphs, mode, normalize=True)
@@ -185,7 +206,6 @@ class Argument(Component):
 
         Args:
             mrphs: A list of morphemes.
-
         """
         content_mrphs = []
         seen_content_word = False
@@ -205,7 +225,6 @@ class Argument(Component):
             mrphs: A list of morphemes.
             mode: A type of token representation, which can take either "mrphs" or "reps".
             normalize: If true, the last content word will be normalized.
-
         """
         assert mode in {'mrphs', 'reps'}
         if mode == 'reps':
@@ -220,21 +239,21 @@ class Argument(Component):
 
     def to_dict(self) -> dict:
         """Convert this object into a dictionary."""
-        return dict((
-            ('surf', self.surf),
-            ('normalized_surf', self.normalized_surf),
-            ('mrphs', self.mrphs),
-            ('normalized_mrphs', self.normalized_mrphs),
-            ('reps', self.reps),
-            ('normalized_reps', self.normalized_reps),
-            ('head_reps', self.head_reps),
-            ('eid', self.eid),
-            ('flag', self.flag),
-            ('sdist', self.sdist),
-            ('adnominal_event_ids', self.adnominal_event_ids),
-            ('sentential_complement_event_ids', self.sentential_complement_event_ids),
-            ('children', self.children)
-        ))
+        return dict(
+            surf=self.surf,
+            normalized_surf=self.normalized_surf,
+            mrphs=self.mrphs,
+            normalized_mrphs=self.normalized_mrphs,
+            reps=self.reps,
+            normalized_reps=self.normalized_reps,
+            head_reps=self.head_reps,
+            eid=self.eid,
+            flag=self.flag,
+            sdist=self.sdist,
+            adnominal_event_ids=self.adnominal_event_ids,
+            sentential_complement_event_ids=self.sentential_complement_event_ids,
+            children=self.children
+        )
 
     def to_string(self) -> str:
         """Convert this object into a string."""
